//
// Created by Ilya on 16.10.2022.
//

#include "connection.h"
#include "string"
#include "iostream"

connection::connection(std::string session) {
    this->session = session;
    this->ws.init_asio();
    auto handle = &connection::on_message;
    this->ws.set_message_handler(websocketpp::lib::bind(
            &connection::on_message,
            this,
            websocketpp::lib::placeholders::_1,
            websocketpp::lib::placeholders::_2
    ));
    this->ws.clear_access_channels(websocketpp::log::alevel::all);
    this->ws.set_tls_init_handler(bind(&connection::on_tls_init, this, websocketpp::lib::placeholders::_1));
    websocketpp::lib::error_code ec;
    client::connection_ptr con = this->ws.get_connection("wss://api.redguy.ru/ws/logs/?accessToken=" + session, ec);
    if (ec) {
        std::cout << "could not create connection because: " << ec.message() << std::endl;
        return;
    }
    this->ws.connect(con);
    this->ws.run();
}

void connection::on_message(websocketpp::connection_hdl, client::message_ptr msg) {
    nlohmann::json json = nlohmann::json::parse(msg->get_payload());
    if (json["type"] == "event") {
        if(json["name"] == "logWrite") {
            std::cout << json["log"]["service"]["name"].get<std::string>() << ":" << json["log"]["category"].get<std::string>() << " | [" << json["log"]["created"].get<std::string>() << "] [" << json["log"]["level"].get<std::string>() << "] " << json["log"]["content"].get<std::string>() << std::endl;
        }
    }
}

context_ptr connection::on_tls_init(websocketpp::connection_hdl) {
    context_ptr ctx = websocketpp::lib::make_shared<boost::asio::ssl::context>(boost::asio::ssl::context::tlsv12);

    try {
        ctx->set_options(boost::asio::ssl::context::default_workarounds |
                         boost::asio::ssl::context::no_sslv2 |
                         boost::asio::ssl::context::no_sslv3 |
                         boost::asio::ssl::context::single_dh_use);


        ctx->set_verify_mode(boost::asio::ssl::verify_none);
    } catch (std::exception &e) {
        std::cout << e.what() << std::endl;
    }
    return ctx;
}
